@@ -1,6 +1,6 @@
 /*
    CheMPS2: a spin-adapted implementation of DMRG for ab initio quantum chemistry
-   Copyright (C) 2013-2015 Sebastian Wouters
+   Copyright (C) 2013-2016 Sebastian Wouters
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,10 +28,11 @@
 #include "Lapack.h"
 #include "MPIchemps2.h"
 
-CheMPS2::Heff::Heff(const SyBookkeeper * denBKIn, const Problem * ProbIn){
+CheMPS2::Heff::Heff(const SyBookkeeper * denBKIn, const Problem * ProbIn, const double dvdson_rtol_in){
 
    denBK = denBKIn;
    Prob = ProbIn;
+   dvdson_rtol = dvdson_rtol_in;
 
 }
 
@@ -39,7 +40,7 @@ CheMPS2::Heff::~Heff(){
 
 }
 
-void CheMPS2::Heff::makeHeff(double * memS, double * memHeff, const Sobject * denS, TensorL *** Ltensors, TensorA **** Atensors, TensorB **** Btensors, TensorC **** Ctensors, TensorD **** Dtensors, TensorS0 **** S0tensors, TensorS1 **** S1tensors, TensorF0 **** F0tensors, TensorF1 **** F1tensors, TensorQ *** Qtensors, TensorX ** Xtensors, int nLower, double ** VeffTilde) const{
+void CheMPS2::Heff::makeHeff(double * memS, double * memHeff, const Sobject * denS, TensorL *** Ltensors, TensorOperator **** Atensors, TensorOperator **** Btensors, TensorOperator **** Ctensors, TensorOperator **** Dtensors, TensorS0 **** S0tensors, TensorS1 **** S1tensors, TensorF0 **** F0tensors, TensorF1 **** F1tensors, TensorQ *** Qtensors, TensorX ** Xtensors, int nLower, double ** VeffTilde) const{
 
    const int indexS = denS->gIndex();
    const bool atLeft  = (indexS==0)?true:false;
@@ -246,7 +247,7 @@ void CheMPS2::Heff::makeHeff(double * memS, double * memHeff, const Sobject * de
 
 }
 
-void CheMPS2::Heff::fillHeffDiag(double * memHeffDiag, const Sobject * denS, TensorC **** Ctensors, TensorD **** Dtensors, TensorF0 **** F0tensors, TensorF1 **** F1tensors, TensorX ** Xtensors, int nLower, double ** VeffTilde) const{
+void CheMPS2::Heff::fillHeffDiag(double * memHeffDiag, const Sobject * denS, TensorOperator **** Ctensors, TensorOperator **** Dtensors, TensorF0 **** F0tensors, TensorF1 **** F1tensors, TensorX ** Xtensors, int nLower, double ** VeffTilde) const{
 
    const int indexS = denS->gIndex();
    const bool atLeft  = (indexS==0)?true:false;
@@ -313,7 +314,7 @@ void CheMPS2::Heff::fillHeffDiag(double * memHeffDiag, const Sobject * denS, Ten
    
 }
 
-double CheMPS2::Heff::SolveDAVIDSON(Sobject * denS, TensorL *** Ltensors, TensorA **** Atensors, TensorB **** Btensors, TensorC **** Ctensors, TensorD **** Dtensors, TensorS0 **** S0tensors, TensorS1 **** S1tensors, TensorF0 **** F0tensors, TensorF1 **** F1tensors, TensorQ *** Qtensors, TensorX ** Xtensors, int nLower, double ** VeffTilde) const{
+double CheMPS2::Heff::SolveDAVIDSON(Sobject * denS, TensorL *** Ltensors, TensorOperator **** Atensors, TensorOperator **** Btensors, TensorOperator **** Ctensors, TensorOperator **** Dtensors, TensorS0 **** S0tensors, TensorS1 **** S1tensors, TensorF0 **** F0tensors, TensorF1 **** F1tensors, TensorQ *** Qtensors, TensorX ** Xtensors, int nLower, double ** VeffTilde) const{
 
    #ifdef CHEMPS2_MPI_COMPILATION
    if ( MPIchemps2::mpi_rank() == MPI_CHEMPS2_MASTER ){
@@ -327,15 +328,16 @@ double CheMPS2::Heff::SolveDAVIDSON(Sobject * denS, TensorL *** Ltensors, Tensor
 
 }
 
-double CheMPS2::Heff::SolveDAVIDSON_main(Sobject * denS, TensorL *** Ltensors, TensorA **** Atensors, TensorB **** Btensors, TensorC **** Ctensors, TensorD **** Dtensors, TensorS0 **** S0tensors, TensorS1 **** S1tensors, TensorF0 **** F0tensors, TensorF1 **** F1tensors, TensorQ *** Qtensors, TensorX ** Xtensors, int nLower, double ** VeffTilde) const{
+double CheMPS2::Heff::SolveDAVIDSON_main(Sobject * denS, TensorL *** Ltensors, TensorOperator **** Atensors, TensorOperator **** Btensors, TensorOperator **** Ctensors, TensorOperator **** Dtensors, TensorS0 **** S0tensors, TensorS1 **** S1tensors, TensorF0 **** F0tensors, TensorF1 **** F1tensors, TensorQ *** Qtensors, TensorX ** Xtensors, int nLower, double ** VeffTilde) const{
 
    int inc1 = 1;
    int veclength = denS->gKappa2index( denS->gNKappa() );
 
-   Davidson deBoskabouter( veclength, CheMPS2::HEFF_DAVIDSON_NUM_VEC,
-                                      CheMPS2::HEFF_DAVIDSON_NUM_VEC_KEEP,
-                                      CheMPS2::HEFF_DAVIDSON_DMRG_RTOL,
-                                      CheMPS2::HEFF_DAVIDSON_PRECOND_CUTOFF, CheMPS2::HEFF_debugPrint );
+   Davidson deBoskabouter( veclength, CheMPS2::DAVIDSON_NUM_VEC,
+                                      CheMPS2::DAVIDSON_NUM_VEC_KEEP,
+                                      // CheMPS2::DAVIDSON_DMRG_RTOL,
+                                      dvdson_rtol,
+                                      CheMPS2::DAVIDSON_PRECOND_CUTOFF, CheMPS2::HEFF_debugPrint );
    double ** whichpointers = new double*[2];
 
    char instruction = deBoskabouter.FetchInstruction( whichpointers );
@@ -384,7 +386,7 @@ double CheMPS2::Heff::SolveDAVIDSON_main(Sobject * denS, TensorL *** Ltensors, T
 }
 
 #ifdef CHEMPS2_MPI_COMPILATION
-double CheMPS2::Heff::SolveDAVIDSON_help(Sobject * denS, TensorL *** Ltensors, TensorA **** Atensors, TensorB **** Btensors, TensorC **** Ctensors, TensorD **** Dtensors, TensorS0 **** S0tensors, TensorS1 **** S1tensors, TensorF0 **** F0tensors, TensorF1 **** F1tensors, TensorQ *** Qtensors, TensorX ** Xtensors, int nLower, double ** VeffTilde) const{
+double CheMPS2::Heff::SolveDAVIDSON_help(Sobject * denS, TensorL *** Ltensors, TensorOperator **** Atensors, TensorOperator **** Btensors, TensorOperator **** Ctensors, TensorOperator **** Dtensors, TensorS0 **** S0tensors, TensorS1 **** S1tensors, TensorF0 **** F0tensors, TensorF1 **** F1tensors, TensorQ *** Qtensors, TensorX ** Xtensors, int nLower, double ** VeffTilde) const{
 
    int veclength = denS->gKappa2index( denS->gNKappa() );
    double * vecin  = new double[ veclength ];
