@@ -24,6 +24,7 @@
 #include <string.h>
 #include <getopt.h>
 #include <sys/stat.h>
+#include <assert.h>
 
 #include "Initialize.h"
 #include "CASSCF.h"
@@ -205,6 +206,41 @@ int clean_exit( const int return_code ){
 
 }
 
+bool print_molcas_reorder( int * dmrg2ham, const int L, const string filename, const bool read ){
+
+   bool on_disk = false;
+
+   if ( read ){
+      struct stat file_info;
+      on_disk = (( filename.length() > 0 ) && ( stat( filename.c_str(), &file_info ) == 0 ));
+      if ( on_disk ){
+         ifstream input( filename.c_str() );
+         string line;
+         getline( input, line );
+         const int num = count( line.begin(), line.end(), ',' ) + 1;
+         assert( num == L );
+         fetch_ints( line, dmrg2ham, L );
+         input.close();
+         cout << "Read orbital reordering = [ ";
+         for ( int orb = 0; orb < L - 1; orb++ ){ cout << dmrg2ham[ orb ] << ", "; }
+         cout << dmrg2ham[ L - 1 ] << " ]." << endl;
+      }
+   } else { // write
+      FILE * capturing;
+      capturing = fopen( filename.c_str(), "w" ); // "w" with fopen means truncate file
+      for ( int orb = 0; orb < L - 1; orb++ ){
+         fprintf( capturing, "%d, ", dmrg2ham[ orb ] );
+      }
+      fprintf( capturing, "%d \n", dmrg2ham[ L - 1 ] );
+      fclose( capturing );
+      cout << "Orbital reordering written to " << filename << "." << endl;
+      on_disk = true;
+   }
+
+   return on_disk;
+
+}
+
 void print_help(){
 
 cout << "\n"
@@ -300,6 +336,9 @@ cout << "\n"
 "       MOLCAS_REORDER = bool\n"
 "              When all orbitals are active orbitals, switch on orbital reordering based on the Fiedler vector of the exchange matrix (TRUE or FALSE; default FALSE).\n"
 "\n"
+"       MOLCAS_MPS = bool\n"
+"              When all orbitals are active orbitals, switch on the creation of MPS checkpoints (TRUE or FALSE; default FALSE).\n"
+"\n"
 "       SCF_STATE_AVG = bool\n"
 "              Switch on state-averaging (TRUE or FALSE; default FALSE).\n"
 "\n"
@@ -391,6 +430,7 @@ int main( int argc, char ** argv ){
    string molcas_f4rdm   = "";
    string molcas_fock    = "";
    bool   molcas_reorder = false;
+   bool   molcas_mps     = false;
 
    bool   scf_state_avg    = false;
    double scf_diis_thr     = 0.0;
@@ -512,6 +552,7 @@ int main( int argc, char ** argv ){
       if ( find_character( &caspt2_orbs,      line, "CASPT2_ORBS",      options2, 2 ) == false ){ return clean_exit( -1 ); }
 
       if ( find_boolean( &molcas_reorder, line, "MOLCAS_REORDER" ) == false ){ return clean_exit( -1 ); }
+      if ( find_boolean( &molcas_mps,     line, "MOLCAS_MPS"     ) == false ){ return clean_exit( -1 ); }
       if ( find_boolean( &scf_state_avg,  line, "SCF_STATE_AVG"  ) == false ){ return clean_exit( -1 ); }
       if ( find_boolean( &caspt2_calc,    line, "CASPT2_CALC"    ) == false ){ return clean_exit( -1 ); }
       if ( find_boolean( &caspt2_checkpt, line, "CASPT2_CHECKPT" ) == false ){ return clean_exit( -1 ); }
@@ -709,6 +750,7 @@ int main( int argc, char ** argv ){
       cout << "   MOLCAS_F4RDM       = " << molcas_f4rdm << endl;
       cout << "   MOLCAS_FOCK        = " << molcas_fock << endl;
       cout << "   MOLCAS_REORDER     = " << (( molcas_reorder ) ? "TRUE" : "FALSE" ) << endl;
+      cout << "   MOLCAS_MPS         = " << (( molcas_mps ) ? "TRUE" : "FALSE" ) << endl;
    } else {
       cout << "   SCF_STATE_AVG      = " << (( scf_state_avg ) ? "TRUE" : "FALSE" ) << endl;
       cout << "   SCF_DIIS_THR       = " << scf_diis_thr << endl;
@@ -759,9 +801,13 @@ int main( int argc, char ** argv ){
       if ( molcas_reorder ){
          int * dmrg2ham = new int[ ham->getL() ];
          if ( am_i_master ){
-            CheMPS2::EdmistonRuedenberg * fiedler = new CheMPS2::EdmistonRuedenberg( ham->getVmat(), group );
-            fiedler->FiedlerGlobal( dmrg2ham );
-            delete fiedler;
+            const bool read_success = (( molcas_mps ) ? print_molcas_reorder( dmrg2ham, ham->getL(), "molcas_fiedler.txt", true ) : false );
+            if ( read_success == false ){
+               CheMPS2::EdmistonRuedenberg * fiedler = new CheMPS2::EdmistonRuedenberg( ham->getVmat(), group );
+               fiedler->FiedlerGlobal( dmrg2ham );
+               delete fiedler;
+               if ( molcas_mps ){ print_molcas_reorder( dmrg2ham, ham->getL(), "molcas_fiedler.txt", false ); }
+            }
          }
          #ifdef CHEMPS2_MPI_COMPILATION
          CheMPS2::MPIchemps2::broadcast_array_int( dmrg2ham, ham->getL(), MPI_CHEMPS2_MASTER );
@@ -770,7 +816,7 @@ int main( int argc, char ** argv ){
          delete [] dmrg2ham;
       }
 
-      CheMPS2::DMRG * dmrgsolver = new CheMPS2::DMRG( prob, opt_scheme, CheMPS2::DMRG_storeMpsOnDisk, tmp_folder );
+      CheMPS2::DMRG * dmrgsolver = new CheMPS2::DMRG( prob, opt_scheme, molcas_mps, tmp_folder );
 
       // Solve for the correct root
       double DMRG_ENERGY;
@@ -805,7 +851,6 @@ int main( int argc, char ** argv ){
       }
 
       // Clean up
-      if ( CheMPS2::DMRG_storeMpsOnDisk ){ dmrgsolver->deleteStoredMPS(); }
       if ( CheMPS2::DMRG_storeRenormOptrOnDisk ){ dmrgsolver->deleteStoredOperators(); }
       delete dmrgsolver;
       delete prob;
